"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const ast_1 = require("./ast");
const module_1 = require("./module");
const ts_morph_1 = require("ts-morph");
const slash_1 = tslib_1.__importDefault(require("slash"));
class Project {
    constructor(src, tsConfigFilePath) {
        this._visitor = new ast_1.CompilerVisitor();
        this._project = new ts_morph_1.Project({
            tsConfigFilePath: tsConfigFilePath,
            addFilesFromTsConfig: true,
        });
        this._project.addSourceFilesAtPaths(src);
    }
    get program() {
        return this._project.getProgram().compilerObject;
    }
    get typeChecker() {
        return this._project.getTypeChecker().compilerObject;
    }
    getSourceFile(file) {
        file = slash_1.default(file);
        const files = this.program.getSourceFiles();
        const main = files.find((it) => it.fileName === file);
        if (main) {
            return module_1.moduleRef.create(main, 'kind', (source, instance) => {
                Reflect.set(instance, '__node', source);
                Reflect.set(instance, '_checker', this.typeChecker);
                Reflect.set(instance, '_program', this.program);
                Reflect.set(instance, '_visitor', this._visitor);
            });
        }
    }
}
exports.Project = Project;
